#!/usr/bin/env python
# -*- coding: utf-8 -*-

# author: Pocholo
# date: 2/25/21

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugins folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.#


from gimpfu import *
import gettext
import os, sys,time 

gettext.install("gimp20", gimp.locale_directory, unicode=True)

def pm_plexi_text_effect(customtext, fontname, size):
    
    #Initiates the temporary state
    pdb.gimp_context_set_defaults()
    pdb.gimp_context_set_default_colors()
	        
    #Background and noise combined
    img = pdb.gimp_image_new(1600, 900, RGB_IMAGE)
    backLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Background", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, backLayer, 0)
    pdb.gimp_layer_add_alpha(backLayer)
    pdb.gimp_context_set_foreground((34, 34, 34)) 
    pdb.gimp_drawable_fill(backLayer, FILL_FOREGROUND)
    pdb.plug_in_hsv_noise(img, backLayer, 2, 3, 4, 4)
    pdb.gimp_display_new(img)
	        
    #Background gradient
    backgrad = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Back gradient", 95, LAYER_MODE_OVERLAY)
    pdb.gimp_image_add_layer(img, backgrad, 0)
    pdb.gimp_layer_add_alpha(backgrad)
    pdb.gimp_drawable_fill(backgrad, FILL_TRANSPARENT)
    pdb.gimp_image_select_ellipse(img, 2, 225, 175, 1150, 550)
    pdb.gimp_edit_bucket_fill(backgrad, BUCKET_FILL_BG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.gimp_selection_none(img)
    pdb.plug_in_gauss(img, backgrad, 400, 400, 1)
    backLayer = pdb.gimp_image_merge_down(img, backgrad, CLIP_TO_IMAGE)
        
	#Create the text		
    textW = size/0.3
    textH = size/0.7
    pdb.gimp_context_set_foreground((255, 255, 255))
    textLayer = pdb.gimp_text_fontname(img, None, 0, 0, customtext, 10, 1, size, PIXELS, fontname)
    pdb.gimp_text_layer_set_justification(textLayer, 2)
    pdb.gimp_text_layer_set_letter_spacing(textLayer, 70)
        
    #Resize text
    pdb.gimp_layer_scale (textLayer, textW, textH, TRUE)
                
    #Making the text centered	
    textLayer = pdb.gimp_image_get_active_layer(img)
    pdb.gimp_image_set_active_layer(img, textLayer)
    wh = pdb.gimp_drawable_width(textLayer)
    hh = pdb.gimp_drawable_height(textLayer)
    w = pdb.gimp_image_width(img)
    h = pdb.gimp_image_height(img)
    pdb.gimp_image_set_active_layer(img,textLayer)
    centerx=w/2
    centery=h/2
    adw=wh/2
    adh=hh/2
    sw=centerx - adw
    sh=centery - adh		
    pdb.gimp_layer_set_offsets(textLayer, sw, sh)
    pdb.gimp_layer_resize_to_image_size(textLayer)
        
    #Rename the text layer and move up
    pdb.gimp_item_set_name(textLayer, "Text")
	
    #using map to create the sphere shape
    pdb.plug_in_map_object(img, textLayer, 
									    0,    # Type of mapping (0=plane,1=sphere,2=box,3=cylinder)
							0.5, 0.5, 2.0,    # viewpoint
							0.5, 0.5, 0.0,    # object pos
							1.0, 0.0, 0.0,    # first axis
							0.0, 1.0, 0.0,    # 2nd axis
						-30.0, 0.0, -50.0,    # axis rotation
										0,    # Type of lightsource (0=point,1=directional,2=none)
						  (255, 255, 255),    # Lightsource color (r,g,b)
						  -0.5, -0.5, 2.0,    # light position
						  -1.0, -1.0, 1.0,    # light direction
			 1.00, 0.60, 0.30, 1.50, 50.0,    # material (amb, diff, refl, spec, high)
									 True,    # antialias
									False,    # tile
									False,    # new image
									 True,    # transparency
									 0.25,    # Sphere/cylinder radius (only used when maptype=1 or 3)
							0.5, 0.5, 0.5,    # box size
									 1.00,    # unused parameter Cylinder length
									 None,    # Box front face (set these to -1 if not used)
									 None,    # Box back face
									 None,    # Box top face									
									 None,    # Box bottom face
									 None,    # Box left face
									 None,    # Box right face
									 None,    # Cylinder top face (unused parameter)
									 None)    # Cylinder bottom face (unused parameter)
			
    #Create first gradien on the text
    pdb.python_layerfx_gradient_overlay(img, textLayer, 
						"Full saturation spectrum CCW",   #gradient 
									                 1,   #gradient_type - bi-linear 
									                 0,   #repeat none 
								                 FALSE,   #reverse 
									                50,   #opacity 
                                                     6,   #mode - differece 
                                                   1200,   #center_x 
                                                   300,   #center_y 
                                                   -60,   #angle 
                                                   500,   #width 
                                                 FALSE)   #merge
        
    #Make the gradient 3d
    textg1Layer = pdb.gimp_image_get_layer_by_name(img, "Text-gradient")
    pdb.script_fu_3d_extrude(img, textg1Layer, 
										     0,  #extrusion type 
										    50,  #extrusion depth 
										    90,  #extrusion direction 
										    90,  #bump azimuth 
										    45,  #bump elevation 
										    65,  #bump depth 
									      0.30,  #bump ambient light 
										     0,  #bump curve 
									     FALSE,  #invert bump 
									     FALSE,  #merge extrusion 
									     FALSE,  #overlay with pattern 
									     FALSE,  #extrude with pattern 
								   "Dried mud")  #extrision pattern)
       
    #Change the layer mode on the text, extrusion and text-gradient layers
    extrusionLayer = pdb.gimp_image_get_layer_by_name(img, "Extrusion")    
    pdb.gimp_layer_set_mode(extrusionLayer, LAYER_MODE_GRAIN_MERGE)
    pdb.gimp_layer_resize_to_image_size(extrusionLayer)
    
    #Create first gradien on the text
    pdb.python_layerfx_gradient_overlay(img, extrusionLayer, 
						     "Full saturation spectrum CW",   #gradient 
									                      0,   #gradient_type - linear 
									                      0,   #repeat none 
                                                      FALSE,   #reverse 
                                                        100,   #opacity 
                                                          4,   #mode - screen 
                                                       1100,   #center_x 
                                                          0,   #center_y 
                                                        -30,   #angle 
                                                       1000,   #width 
                                                      FALSE)   #merge
        
    
    #Move the text-gradient 25x down
    textg1Layer = pdb.gimp_image_get_layer_by_name(img, "Text-gradient")   
    textg1Layer = pdb.gimp_item_transform_translate(textg1Layer, 0, 25)
    pdb.gimp_layer_resize_to_image_size(textg1Layer)
    
    #Create first gradient on the text
    pdb.python_layerfx_gradient_overlay(img, textLayer, 
						 "Full saturation spectrum CCW",   #gradient 
									                 1,   #gradient_type - bi-linear 
									                 0,   #repeat none 
								                 FALSE,   #reverse 
									                50,   #opacity 
                                                     0,   #mode - normal 
                                                   800,   #center_x 
                                                   600,   #center_y 
                                                   -60,   #angle 
                                                   500,   #width 
                                                  TRUE)   #merge
    
    #Create a shine on the text    
    shadows_lights = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Gleam", 0, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, shadows_lights, 0)
    pdb.gimp_layer_add_alpha(shadows_lights)
    pdb.gimp_drawable_fill(shadows_lights, FILL_TRANSPARENT)
    pdb.gimp_context_set_foreground((255, 255, 255))
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Text") 
    pdb.gimp_image_select_item(img, 2, textLayer)
    pdb.gimp_edit_bucket_fill(shadows_lights, BUCKET_FILL_BG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)  
    pdb.gimp_selection_none(img)
    
    #Gradient overlay
    pdb.python_layerfx_gradient_overlay(img, shadows_lights, 
						             "Flare Glow Angular 1",   #gradient 
                                                          0,   #gradient_type - linear 
                                                          0,   #repeat none 
                                                      FALSE,   #reverse 
                                                         60,   #opacity 
                                                          0,   #mode - normal 
                                                       1200,   #center_x 
                                                        350,   #center_y 
                                                        110,   #angle 
                                                        900,   #width 
                                                      FALSE)   #merge
    
    #create the text spotlight
    non_empty = pdb.gimp_edit_copy(extrusionLayer)
    spotLayer= pdb.gimp_layer_new(img, img.width, img.height, RGB, "Spotlight", 100, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_image_add_layer(img, spotLayer, 5)
    pdb.gimp_layer_add_alpha(spotLayer)
    pdb.gimp_drawable_fill(spotLayer, FILL_TRANSPARENT)
    spotLayer = pdb.gimp_edit_paste(spotLayer, FALSE)
    pdb.gimp_floating_sel_anchor(spotLayer)  
    spotLayer = pdb.gimp_image_get_layer_by_name(img, "Spotlight") 
    pdb.gimp_image_select_item(img, 2, spotLayer)
    pdb.gimp_context_set_foreground((255, 255, 255))
    pdb.gimp_edit_bucket_fill(spotLayer, BUCKET_FILL_BG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)  
    pdb.gimp_selection_none(img)
    pdb.plug_in_gauss_rle2(img, spotLayer, 250, 250)
    
    #Plexi text drop shadow
    pdb.python_layerfx_drop_shadow(img, extrusionLayer, 
                                            ( 0, 0, 0),   #color
                                                    30,   #opacity
                                                     0,   #contou - linear
                                                     0,   #noise 
                                                     3,   #mode for multiply 
                                                     0,   #spread 
                                                    10,   #size 
                                                    90,   #offset_angle 
                                                    10,   #offset_distance 
                                                  TRUE,   #knockout 
                                                 FALSE)   #merge
        
    #Set gimp to default
    pdb.gimp_context_set_defaults()
    pdb.gimp_context_set_default_colors()
    pdb.gimp_displays_flush()
	
register(
    "pm_plexi_text_effect",
    "Creates a plexiglass text",
    "Creates a plexiglass text",
    "Pocholo",
    "Pocholo",
    "2021",
    "Plexi text effect",
    "",
    [        
    (PF_STRING, "customtext", "Text string", 'PLEXI'),
    (PF_FONT, "font", "Font name", "Bauhaus 93,"),
    (PF_SPINNER, "size", "Text size", 350, (300, 350, 1)),
    ],
    [],
    pm_plexi_text_effect, menu="<Image>/Pocholo-scripts/Plexi text effect",
    domain=("gimp20", gimp.locale_directory))
main()			